@@ -1,11 +1,10 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 #include <c10/core/QScheme.h>
 
 // Forward declarations of core ATen types used in dispatch functions
 namespace c10 {
 
-template<typename T>
-class optional;
 template<typename T>
 class List;
 template<typename T>
@@ -35,6 +34,7 @@ using IOptTensorListRef = c10::IListRef<OptionalTensorRef>;
 using DimnameList = c10::ArrayRef<Dimname>;
 using IntArrayRef = c10::ArrayRef<int64_t>;
 using OptionalIntArrayRef = c10::OptionalArrayRef<int64_t>;
+using OptionalSymIntArrayRef = c10::OptionalArrayRef<c10::SymInt>;
 
 using c10::Stream;
 using c10::Storage;
@@ -45,3 +45,7 @@ using c10::SymIntList;
 using c10::TensorOptions;
 
 }  // namespace at
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
