@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 // DO NOT DEFINE STATIC DATA IN THIS HEADER!
@@ -830,3 +831,7 @@ static inline Vectorized<T> binary_op_as_fp32(
 #endif // CPU_CAPABILITY_AVX2
 } // namespace CPU_CAPABILITY
 } // namespace at::vec
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
