@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #ifndef THCP_EVENT_INC
 #define THCP_EVENT_INC
 
@@ -17,3 +18,7 @@ inline bool THCPEvent_Check(PyObject* obj) {
 }
 
 #endif // THCP_EVENT_INC
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
