@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <array>
@@ -73,3 +74,7 @@ using namespace torch::aot_inductor;
 [[maybe_unused]] inline int64_t align(int64_t nbytes) {
   return (nbytes + 64 - 1) & -64;
 }
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
