@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 #include <cstddef>
 
@@ -17,3 +18,7 @@ constexpr size_t BYTECODE_INDEX_ARGUMENT_DEFAULT_VALUE = 2;
 
 constexpr size_t BYTECODE_INDEX_MODULE_DEBUG_HANDLES = 0;
 } // namespace torch::jit
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
