@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/csrc/profiler/stubs/base.h>
@@ -20,3 +21,7 @@ std::tuple<std::string, uint64_t> getShaderNameAndDurationNs(
     const vulkan_id_t& vulkan_id);
 
 } // namespace torch::profiler::impl::vulkan
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
