@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #include <c10/util/irange.h>
 #include <torch/csrc/profiler/unwind/debug_info.h>
 #include <torch/csrc/profiler/unwind/dwarf_enums.h>
@@ -326,3 +327,7 @@ struct LineNumberProgram {
 };
 
 } // namespace torch::unwind
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
