@@ -581,42 +581,6 @@ def _sfdp_replacement_20(query, key, value, attn_mask, dropout_p):
     )
 
 
-def _sfdp_pattern_24(query, key, value, attention_mask):
-    """
-    this pattern is for MBartForCausalLM/PLBartForCausalLM.
-    attn_mask has a different dtype with QKV.
-    there is no scale in sdpa.
-    """
-    bs = query.size(0)
-    n_head = query.size(1)
-    seq_len = query.size(2)
-    head_size = query.size(3)
-    q = query.view(bs * n_head, -1, head_size)
-    k = key.reshape(bs * n_head, -1, head_size)
-    v = value.reshape(bs * n_head, -1, head_size)
-    attn_weights = torch.bmm(q, k.transpose(1, 2))
-    attn_weights = attn_weights.view(bs, n_head, seq_len, -1) + attention_mask
-    attn_weights = attn_weights.view(bs * n_head, seq_len, -1)
-    attn_weights = torch.nn.functional.softmax(attn_weights, dim=-1)
-    if query.dtype == torch.half:
-        attn_weights = attn_weights.to(torch.half)
-    attn_output = torch.bmm(attn_weights, v)
-    attn_output = attn_output.view(bs, n_head, seq_len, head_size)
-    return attn_output
-
-
-def _sfdp_replacement_24(query, key, value, attention_mask):
-    counters["inductor"]["fuse_attention"] += 1
-    return _scaled_dot_product_attention(
-        query,
-        key,
-        value,
-        attn_mask=attention_mask.to(dtype=query.dtype),
-        is_causal=False,
-        scale=1,
-    )
-
-
 def _sfdp_pattern_21(query, key, value, attn_mask):
     # for T5 with inplace add
     query = query.permute([0, 2, 1, 3])
@@ -643,7 +607,7 @@ def _sfdp_replacement_21(query, key, value, attn_mask):
         query,
         key,
         value,
-        attn_mask=attn_mask,
+        attn_mask=attn_mask.to(dtype=query.dtype),
         is_causal=False,
         scale=1.0,
     )
@@ -676,7 +640,7 @@ def _sfdp_replacement_22(query, key, value, attn_mask):
             query,
             key,
             value,
-            attn_mask=attn_mask,
+            attn_mask=attn_mask.to(dtype=query.dtype),
             is_causal=False,
             scale=1.0,
         ),
@@ -723,6 +687,42 @@ def _sfdp_replacement_23(query, key, value):
     )
 
 
+def _sfdp_pattern_24(query, key, value, attention_mask):
+    """
+    this pattern is for MBartForCausalLM/PLBartForCausalLM.
+    attn_mask has a different dtype with QKV.
+    there is no scale in sdpa.
+    """
+    bs = query.size(0)
+    n_head = query.size(1)
+    seq_len = query.size(2)
+    head_size = query.size(3)
+    q = query.view(bs * n_head, -1, head_size)
+    k = key.reshape(bs * n_head, -1, head_size)
+    v = value.reshape(bs * n_head, -1, head_size)
+    attn_weights = torch.bmm(q, k.transpose(1, 2))
+    attn_weights = attn_weights.view(bs, n_head, seq_len, -1) + attention_mask
+    attn_weights = attn_weights.view(bs * n_head, seq_len, -1)
+    attn_weights = torch.nn.functional.softmax(attn_weights, dim=-1)
+    if query.dtype == torch.half:
+        attn_weights = attn_weights.to(torch.half)
+    attn_output = torch.bmm(attn_weights, v)
+    attn_output = attn_output.view(bs, n_head, seq_len, head_size)
+    return attn_output
+
+
+def _sfdp_replacement_24(query, key, value, attention_mask):
+    counters["inductor"]["fuse_attention"] += 1
+    return _scaled_dot_product_attention(
+        query,
+        key,
+        value,
+        attn_mask=attention_mask.to(dtype=query.dtype),
+        is_causal=False,
+        scale=1,
+    )
+
+
 def _sfdp_params_check(match):
     assert all(k in match.kwargs for k in ("query", "key", "value"))
     query = match.kwargs["query"].meta["val"]
@@ -1024,6 +1024,13 @@ def _get_sfdp_patterns():
                 {},
                 _sfdp_params_check,
             ),
+            (
+                _sfdp_pattern_21,
+                _sfdp_replacement_21,
+                [g_bs1(), g_bs1(), g_bs1(), m_bs1_float()],
+                {},
+                _sfdp_params_check,
+            ),
             (
                 _sfdp_pattern_22,
                 _sfdp_replacement_22,
@@ -1031,6 +1038,13 @@ def _get_sfdp_patterns():
                 {},
                 _sfdp_params_check,
             ),
+            (
+                _sfdp_pattern_22,
+                _sfdp_replacement_22,
+                [g_bs1(), g_bs1(), g_bs1(), m_bs1_float()],
+                {},
+                _sfdp_params_check,
+            ),
             (
                 _sfdp_pattern_23,
                 _sfdp_replacement_23,
@@ -1038,6 +1052,13 @@ def _get_sfdp_patterns():
                 {},
                 _sfdp_params_check,
             ),
+            (
+                _sfdp_pattern_23,
+                _sfdp_replacement_23,
+                [g_bs1(), g_bs1(), g_bs1()],
+                {},
+                _sfdp_params_check,
+            ),
             (
                 _sfdp_pattern_24,
                 _sfdp_replacement_24,
