@@ -1,6 +1,6 @@
 import threading
 from collections.abc import Sequence
-from typing import Any, cast, Optional, Union
+from typing import Any, cast
 
 import torch
 from torch._utils import ExceptionWrapper
@@ -12,8 +12,8 @@ __all__ = ["get_a_var", "parallel_apply"]
 
 
 def get_a_var(
-    obj: Union[torch.Tensor, list[Any], tuple[Any, ...], dict[Any, Any]],
-) -> Optional[torch.Tensor]:
+    obj: torch.Tensor | list[Any] | tuple[Any, ...] | dict[Any, Any],
+) -> torch.Tensor | None:
     if isinstance(obj, torch.Tensor):
         return obj
 
@@ -31,8 +31,8 @@ def get_a_var(
 def parallel_apply(
     modules: Sequence[Module],
     inputs: Sequence[Any],
-    kwargs_tup: Optional[Sequence[dict[str, Any]]] = None,
-    devices: Optional[Sequence[Optional[Union[int, torch.device]]]] = None,
+    kwargs_tup: Sequence[dict[str, Any]] | None = None,
+    devices: Sequence[int | torch.device | None] | None = None,
 ) -> list[Any]:
     r"""Apply each `module` in :attr:`modules` in parallel on each of :attr:`devices`.
 
@@ -46,9 +46,9 @@ def parallel_apply(
     element of :attr:`inputs` can either be a single object as the only argument
     to a module, or a collection of positional arguments.
     """
-    assert len(modules) == len(
-        inputs
-    ), f"The number of modules {len(modules)} is not equal to the number of inputs {len(inputs)}"
+    assert len(modules) == len(inputs), (
+        f"The number of modules {len(modules)} is not equal to the number of inputs {len(inputs)}"
+    )
     if kwargs_tup is not None:
         assert len(modules) == len(kwargs_tup)
     else:
@@ -58,7 +58,9 @@ def parallel_apply(
     else:
         devices = [None] * len(modules)
     devices = [_get_device_index(x, True) for x in devices]
-    streams = [torch.cuda.current_stream(x) for x in devices]
+    streams = [torch.accelerator.current_stream(x) for x in devices]
+    assert torch.accelerator.is_available(), "No available accelerator found."
+    device_type = torch.accelerator.current_accelerator().type  # type: ignore[union-attr]
     lock = threading.Lock()
     results = {}
     grad_enabled, autocast_enabled = (
@@ -71,8 +73,8 @@ def parallel_apply(
         module: Module,
         input: Any,
         kwargs: dict[str, Any],
-        device: Optional[Union[int, torch.device]] = None,
-        stream: Optional[torch.cuda.Stream] = None,
+        device: int | torch.device | None = None,
+        stream: torch.Stream | None = None,
     ) -> None:
         torch.set_grad_enabled(grad_enabled)
         if device is None:
@@ -85,12 +87,16 @@ def parallel_apply(
                     )
                 return
             device = t.get_device()
+        if isinstance(device, torch.device):
+            device = device.index
         if stream is None:
-            stream = torch.cuda.current_stream(device)
+            stream = torch.accelerator.current_stream(device)
         try:
-            with torch.cuda.device(device), torch.cuda.stream(
-                stream
-            ), torch.amp.autocast("cuda", enabled=autocast_enabled):
+            with (
+                torch.accelerator.device_index(device),
+                stream,
+                torch.amp.autocast(device_type, enabled=autocast_enabled),
+            ):
                 # this also avoids accidental slicing of `input` if it is a Tensor
                 if not isinstance(input, (list, tuple)):
                     input = (input,)
@@ -109,7 +115,7 @@ def parallel_apply(
                 target=_worker, args=(i, module, input, kwargs, device, stream)
             )
             for i, (module, input, kwargs, device, stream) in enumerate(
-                zip(modules, inputs, kwargs_tup, devices, streams)
+                zip(modules, inputs, kwargs_tup, devices, streams, strict=True)
             )
         ]
 
